"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const api_core_1 = require("api-core");
const ApiQueryStringParser_1 = require("./ApiQueryStringParser");
class RedisApiRouter {
    constructor(apis) {
        this.apply = (pubClient, subClient, channel = 'api-core') => {
            subClient.subscribe(channel, (err) => {
                if (err)
                    console.error(err.stack);
            });
            const publish = (data) => {
                pubClient.publish(channel, JSON.stringify(data));
            };
            const messages = {
                400: `Unable to parse incoming message on channel '${channel}'.`,
                422: `Invalid incoming message received on channel '${channel}'.`
            };
            const publishError = (code) => {
                console.error(messages[400]);
                publish({ type: 'error', code, message: messages[code] });
            };
            subClient.on("message", (incomingChannel, message) => {
                if (channel !== incomingChannel)
                    return;
                let data;
                try {
                    data = JSON.parse(message);
                }
                catch (e) {
                    publishError(400);
                    return;
                }
                if (data.type !== 'request') {
                    publishError(422);
                    return;
                }
                let API = this.defaultApi;
                if (data.version) {
                    const index = this.apiVersions.indexOf(data.version);
                    if (index === -1) {
                        publishError(422);
                        return;
                    }
                    else {
                        API = this.apis[index];
                    }
                }
            });
            app.use((req, res, next) => {
                if (req.error || !req.api)
                    next();
                else {
                    try {
                        let request = req.api.parseRequest(req.apiPath.split('/'));
                        if (!request.path.segments.length) {
                            req.error = new api_core_1.ApiEdgeError(404, 'Not Found');
                            return next();
                        }
                        request.context = ApiQueryStringParser_1.ApiQueryStringParser.parse(req.query, request.path);
                        if (req.body) {
                            request.body = req.body;
                        }
                        switch (req.method) {
                            case "GET":
                                request.type = api_core_1.ApiRequestType.Read;
                                break;
                            case "POST":
                                request.type = api_core_1.ApiRequestType.Create;
                                break;
                            case "PUT":
                                request.type = api_core_1.ApiRequestType.Update;
                                break;
                            case "PATCH":
                                request.type = api_core_1.ApiRequestType.Patch;
                                break;
                            case "DELETE":
                                request.type = api_core_1.ApiRequestType.Delete;
                                break;
                        }
                        let query = req.api.buildQuery(request);
                        query.request = request;
                        query.execute(req.user)
                            .then((resp) => {
                            if (resp.metadata) {
                                if (resp.metadata.pagination) {
                                    const total = resp.metadata.pagination.total || 0, limit = +req.query.limit || ApiQueryStringParser_1.ApiQueryStringParser.defaultLimit;
                                    res.setHeader('X-Total-Count', req.query.page ? Math.ceil(total / limit) : total);
                                }
                            }
                            res.json(resp.data);
                        })
                            .catch((e) => {
                            req.error = e;
                            next();
                        });
                    }
                    catch (e) {
                        req.error = e;
                        next();
                    }
                }
            });
            app.use((req, res) => {
                let e = req.error;
                if (e instanceof api_core_1.ApiEdgeError) {
                    res.status(e.status).send(e.message);
                }
                else {
                    res.status(500).send("Internal Server Error");
                }
            });
        };
        this.apis = apis;
        this.defaultApi = apis[0];
        this.apiVersions = apis.map(api => api.version);
    }
}
exports.RedisApiRouter = RedisApiRouter;
//# sourceMappingURL=RedisApiRouter.js.map