import {ApiEdgeError, ApiEdgeQueryResponse, Api, ApiRequestType} from "api-core";
import {ApiQueryStringParser} from "./ApiQueryStringParser";
import * as redis from "redis";

interface RedisProviderRequest {
    type: 'request'
    version?: string
}

export class RedisApiRouter {

    defaultApi: Api;
    apis: Api[];

    private apiVersions: string[];

    constructor(apis: Api[]) {
        this.apis = apis;
        this.defaultApi = apis[0];

        this.apiVersions = apis.map(api => api.version);
    }

    apply = (pubClient: redis.RedisClient, subClient: redis.RedisClient, channel = 'api-core') => {

        subClient.subscribe(channel, (err) => {
            //TODO: Handle errors
            if(err) console.error(err.stack)
        });

        const publish = (data: any) => {
            pubClient.publish(channel, JSON.stringify(data))
        };

        const messages: { [key: number]: string } = {
            400: `Unable to parse incoming message on channel '${channel}'.`,
            422: `Invalid incoming message received on channel '${channel}'.`
        };

        const publishError = (code: number) => {
            console.error(messages[400]);
            publish({ type: 'error', code, message: messages[code] })
        };


        subClient.on("message", (incomingChannel, message) => {
            if(channel !== incomingChannel) return;

            let data: RedisProviderRequest;
            try {
                data = JSON.parse(message)
            }
            catch(e) {
                publishError(400);
                return
            }

            if(data.type !== 'request') {
                publishError(422);
                return
            }

            let API = this.defaultApi;
            if(data.version) {
                const index = this.apiVersions.indexOf(data.version);
                if (index === -1) {
                    publishError(422);
                    return
                }
                else {
                    API = this.apis[index];
                }
            }




        });


        app.use((req: ExtendedRequest, res: express.Response, next: express.NextFunction) => {
            if(req.error || !req.api) next();
            else {
                try {
                    let request = req.api.parseRequest(req.apiPath.split('/'));

                    if(!request.path.segments.length) {
                        req.error = new ApiEdgeError(404, 'Not Found');
                        return next()
                    }

                    request.context = ApiQueryStringParser.parse(req.query, request.path);

                    if (req.body) {
                        request.body = req.body;
                    }

                    switch(req.method) {
                        case "GET":
                            request.type = ApiRequestType.Read;
                            break;
                        case "POST":
                            request.type = ApiRequestType.Create;
                            break;
                        case "PUT":
                            request.type = ApiRequestType.Update;
                            break;
                        case "PATCH":
                            request.type = ApiRequestType.Patch;
                            break;
                        case "DELETE":
                            request.type = ApiRequestType.Delete;
                            break;
                    }

                    let query = req.api.buildQuery(request);
                    query.request = request;

                    //TODO: req.user - Is this an acceptable solution?
                    query.execute(req.user)
                        .then((resp: ApiEdgeQueryResponse) => {
                            if(resp.metadata) {
                                if(resp.metadata.pagination) {
                                    const total = resp.metadata.pagination.total || 0,
                                        limit = +req.query.limit || ApiQueryStringParser.defaultLimit;
                                    res.setHeader('X-Total-Count', req.query.page ? Math.ceil(total / limit) : total);
                                }
                            }

                            res.json(resp.data)
                        })
                        .catch((e: any) => {
                            req.error = e;
                            next()
                        })
                }
                catch (e) {
                    req.error = e;
                    next()
                }
            }
        });

        app.use((req: ExtendedRequest, res: express.Response) => {
            let e = req.error;
            if(e instanceof ApiEdgeError) {
                res.status(e.status).send(e.message)
            }
            else {
                res.status(500).send("Internal Server Error")
            }
        });
    }
}